SystemOrganization addCategory: #'SqueakNxt-EtoysCompiler'!

RBProgramNodeVisitor subclass: #NxtPlayerTranslator
	instanceVariableNames: 'programName motors sensors source morph programClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-EtoysCompiler'!

!NxtPlayerTranslator class methodsFor: 'as yet unclassified' stamp: 'Richo 1/4/2011 17:13'!
on: aMorph
	^(self basicNew setMorph: aMorph) initialize! !

!NxtPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 1/4/2011 17:24'!
acceptMessageNode: aMessageNode
	| selector |
	selector := aMessageNode selector.
	(SqueakNxtPlayer selectors includes: selector) 
		ifTrue: [(self rules at: selector ifAbsent: [self error: selector printString, ': Not supported or invalid']) value: aMessageNode].
	(self etoysMessages includesKey: selector)
		ifTrue: [(self etoysMessages at: selector) value: aMessageNode].
	super acceptMessageNode: aMessageNode! !

!NxtPlayerTranslator methodsFor: 'as yet unclassified' stamp: 'Richo 1/4/2011 17:25'!
collectMotorsAndSensors
	"Finds out which motor and sensor morphs are connected to the nxt brick.
	It's not the fastest way (nor the cleanest) but it works..."
	| nxt |
	motors := Array new: 3.
	sensors := Array new: 4.
	nxt := morph nxt.
	NxtPluggableDeviceMorph allSubInstancesDo: [:m | 
		m device port caseOf: {
			[nxt portA] -> [motors at: 1 put: m].
			[nxt portB] -> [motors at: 2 put: m].
			[nxt portC] -> [motors at: 3 put: m].
			[nxt port1] -> [sensors at: 1 put: m].
			[nxt port2] -> [sensors at: 2 put: m].
			[nxt port3] -> [sensors at: 3 put: m].
			[nxt port4] -> [sensors at: 4 put: m].
		} otherwise: []
	]
! !

!NxtPlayerTranslator methodsFor: 'as yet unclassified' stamp: 'Richo 1/4/2011 17:26'!
compileMainMethod: scriptName
	self programClass compileSilently: 'main
	self setUp.
	self wait: 500.
	[self ', scriptName,'] repeat' classified: #main! !

!NxtPlayerTranslator methodsFor: 'as yet unclassified' stamp: 'Richo 1/4/2011 17:27'!
compileSetUpMethod
| methodSource |
methodSource := WriteStream on: ''.
methodSource nextPutAll: 'setUp'; cr.
sensors doWithIndex: [:m :index |
	m isNil ifFalse: [ | zeroBasedIndex |
		zeroBasedIndex := (index - 1) asString.
		methodSource nextPutAll: '	self setSensor'.
		m device type caseOf: {
			[SensorType lowspeed9V] -> [methodSource nextPutAll: 'LowSpeed: ', zeroBasedIndex].
			[SensorType lightActive] -> [methodSource nextPutAll: 'Light: ', zeroBasedIndex, ' active: true'].
			[SensorType lightInactive] -> [methodSource nextPutAll: 'Light: ', zeroBasedIndex, ' active: false'].
			[SensorType soundDba] -> [methodSource nextPutAll: 'Sound: ', zeroBasedIndex, ' scaling: false'].
			[SensorType soundDb] -> [methodSource nextPutAll: 'Sound: ', zeroBasedIndex, ' scaling: true'].
			[SensorType switch] -> [methodSource nextPutAll: 'Touch: ', zeroBasedIndex]
		} otherwise: [].
		methodSource nextPut: $.; cr.
	]
].
self programClass compileSilently: methodSource contents classified: 'setUp'! !

!NxtPlayerTranslator methodsFor: 'as yet unclassified' stamp: 'Richo 1/4/2011 18:09'!
createNxtProgramSubclass
	"self programClass removeFromSystem.
	NxtProgram subclass: self programName
		instanceVariableNames: ''
		instanceVariableTypes: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'NxtPrograms'"
	SystemChangeNotifier uniqueInstance doSilently: [
	programClass isNil 
		ifTrue: [programClass := (NxtProgram newUniqueClassInstVars: '' classInstVars: '')]
		ifFalse: [programClass selectors do: [:each | programClass removeSelector: each]]]
! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/4/2011 17:07'!
etoysMessages
"I don't know why but recent versions of Etoys use this weird messages instead of the operators."
^ {
#eToysLT: -> [:node | node selector: #<].
#eToysLE: -> [:node | node selector: #<=].
#eToysEQ: -> [:node | node selector: #=].
#eToysNE: -> [:node | node selector: #~=].
#eToysGT: -> [:node | node selector: #>].
#eToysGE: -> [:node | node selector: #>=].
#isDivisibleBy: -> [:node | self error: '#isDivisibleBy: Not supported yet'].
} as: Dictionary! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/4/2011 11:39'!
indexFor: aVariableNode in: aCollection
	| index |
	index := 0.
	aCollection do: [:each | 
	index := index + 1.
(each notNil and: [each assuredPlayer uniqueNameForReference = aVariableNode token value]) ifTrue: [^self literal: index - 1]].
	^ self error: 'Index not found'! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/4/2011 11:23'!
literal: aLiteralObject
	^ RBLiteralValueNode value: aLiteralObject! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/4/2011 17:58'!
programClass
	"^ Smalltalk at: self programName"
	^ programClass! !

!NxtPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 1/4/2011 17:22'!
rules
"These are the rules to modify the parse tree of the SqueakNxtPlayer scripts"
| selfVar |
selfVar := RBVariableNode named: 'self'.
^ {
#getIsPressed -> [:node | 
	node arguments: {self indexFor: node receiver in: sensors};
		 selector: #sensorBoolean:;
		receiver: selfVar].
#playTone: -> [:node | 
	node receiver: selfVar;
		  arguments: node arguments, {self literal: 200};
		  selector: #playTone:duration:].
#setPower: -> [:node | 
	node arguments: {self indexFor: node receiver in: motors} , node arguments;
		selector: #onFwd:speed:;
		receiver: selfVar].
#getAudible -> [:node | | copy |
	copy := node copy.
	node receiver: copy; arguments: {self literal: SensorType soundDba}; selector: #=.
	copy arguments: {self indexFor: copy receiver in: sensors}; selector: #sensorType:; receiver: selfVar].
#getActive -> [:node | | copy |
	copy := node copy.
	node receiver: copy; arguments: {self literal: SensorType lightActive}; selector: #=.
	copy arguments: {self indexFor: copy receiver in: sensors}; selector: #sensorType:; receiver: selfVar].
#getDistanceCm -> [:node |
	node arguments: {self indexFor: node receiver in: sensors};
		 selector: #sensorUS:;
		receiver: selfVar].
#getValue -> [:node |
	node arguments: {self indexFor: node receiver in: sensors};
			selector: #sensorValue:;
			receiver: selfVar].
#float -> [:node |
	node arguments: {self indexFor: node receiver in: motors};
		selector: #float:;
		receiver: selfVar].
#brake -> [:node |
	node arguments: {self indexFor: node receiver in: motors};
		selector: #off:;
		receiver: selfVar].
#playSoundFile: -> [:node | node selector: #playFile:; receiver: selfVar].
#getPower -> [:node |
	node arguments: {self indexFor: node receiver in: motors};
		selector: #motorPower:;
		receiver: selfVar].
#setActive: -> [:node |
	node arguments: {self indexFor: node receiver in: sensors}, node arguments;
		 selector: #setSensorLight:active:;
		receiver: selfVar].
#setAudible: -> [:node |
	node arguments: {self indexFor: node receiver in: sensors}, {RBMessageNode receiver: node arguments first selector: #not};
		 selector: #setSensorSound:scaling:;
		receiver: selfVar]
} as: Dictionary! !

!NxtPlayerTranslator methodsFor: 'private' stamp: 'Richo 1/4/2011 17:14'!
setMorph: aMorph
	morph := aMorph ! !

!NxtPlayerTranslator methodsFor: 'as yet unclassified' stamp: 'Richo 1/4/2011 18:00'!
translateMainScript: scriptName
	| playerClass |
	self createNxtProgramSubclass;
		 collectMotorsAndSensors.
	self compileSetUpMethod.
	playerClass := morph assuredPlayer class.
	playerClass selectors do: [:each | | node |
		self visitNode: (node := (playerClass parseTreeFor: each)).
		self programClass compileSilently: node formattedCode classified: #scripts
		].
	self compileMainMethod: scriptName.
	^ self programClass asNxcProgram: scriptName.! !
